/*
	BromScript - On Runtime Scripting Language.
    Copyright (C) 2012 - 2013  Alex Brouwer (Bromvlieg: http://brom.4o3.nl/)

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "ArgumentData.h"

using namespace Scratch;

namespace BromScript {
	extern int _adcount = 0;

	int ArgumentData::GetCount(){
		return _adcount;
	}

	ArgumentData::ArgumentData(void){
		_adcount++;

		this->Caller = null;
		this->BromScript = null;
		this->Count = 0;
	}

	ArgumentData::~ArgumentData(void){
		_adcount--;
	}

	Variable::VariableType ArgumentData::GetType(int index){
		if (index >= this->Count) return Variable::VariableType::Null;
		return this->Vars[index]->Type;
	}
	
	bool ArgumentData::ErrorOccured(void){
		return this->BromScript->GetCurrentFunction()->ForceReturn;
	}

	bool ArgumentData::CheckType(int index, Variable::VariableType type){
		if (index >= this->Count) return false;
		return this->Vars[index]->Type == type;
	}

	bool ArgumentData::CheckType(int index, Variable::VariableType type, bool throwerror){
		if (index >= this->Count){
			if (throwerror)
				this->BromScript->Error(CString("Missing argument!, Expected ") + Converter::TypeToString(type) + " at argument index " + CString((double)index));
			
			return false;
		}

		if (this->Vars[index]->Type != type){
			if (throwerror)
				this->BromScript->Error(CString("Invalid type given as argument!, Expected ") + Converter::TypeToString(type) + " Got " + Converter::TypeToString(this->Vars[index]->Type));
			
			return false;
		}

		return true;
	}
	
	const char* ArgumentData::GetString(int index){
		if (index >= this->Count) return null;
		if (!this->CheckType(index, Variable::VariableType::String)) return null;

		return *(CString*)this->Vars[index]->Value;
	}
	
	double ArgumentData::GetNumber(int index){
		if (index >= this->Count) return 0;
		if (!this->CheckType(index, Variable::VariableType::Number)) return 0;

		return *(double*)this->Vars[index]->Value;
	}

	bool ArgumentData::GetBool(int index){
		if (index >= this->Count) return false;
		if (!this->CheckType(index, Variable::VariableType::Bool)) return false;

		return this->Vars[index]->Value != null;
	}
	
	Table* ArgumentData::GetTable(int index){
		if (index >= this->Count) return null;
		if (!this->CheckType(index, Variable::VariableType::Table)) return null;

		return (Table*)this->Vars[index]->Value;
	}

	Function* ArgumentData::GetFunction(int index){
		if (index >= this->Count) return null;
		if (!this->CheckType(index, Variable::VariableType::Function)) return null;

		return (Function*)this->Vars[index]->Value;
	}

	void* ArgumentData::GetUserdata(int index, int datatype){
		if (index >= this->Count) return null;
		if (!this->CheckType(index, (Variable::VariableType)datatype)) return null;

		return ((Userdata*)(this->Vars[index]->Value))->TypeData;
	}

	Variable* ArgumentData::GetVariable(int index){
		if (index >= this->Count) return null;

		return this->Vars[index];
	}

	void ArgumentData::SetVariableData(List<Variable*>* vars){
		this->Count = vars->Count;
		this->Vars.Clear();
		for (int i = 0; i < vars->Count; i++)
			this->Vars.Add(vars->Get(i));
	}

	void ArgumentData::InsertVariable(int pos, Variable* var){
		this->Count++;
		this->Vars.Insert(pos, var);
	}

	void ArgumentData::AddVariable(Variable* var){
		this->Count++;
		this->Vars.Add(var);
	}

	void ArgumentData::Error(CString msg){
		this->BromScript->Error(msg);
	}
}
